unit Tuples;

interface

uses
	Classes,
	DBTables,
	Forms,
  	Grids,
	IniFiles,
	Messages,
	SysUtils,
	WinProcs,
	WinTypes;

type
	TTupleList = class(TStringList)
		private
			function GetKey(const Index: Integer): String;
			function GetInt(const Key: String): LongInt;
		public
			function IndexOfKey(const Key: String): Integer;
			procedure ParseKeys(const Params: String; const Delimit: Char);
        	property IntValues[const Key: String]: LongInt read GetInt;
			property Keys[const Index: Integer]: String read GetKey;
	end;

	TTupleGrid = class(TStringGrid)
		procedure AddPair(const Key: String; const Value: String);
		procedure ShowPairs(const Pairs: TTupleList);
	end;

procedure Register;

implementation

{ TTupleList implementations }

function TTupleList.IndexOfKey(const Key: String): Integer;
var
	i: Integer;
begin
	Result := -1;
	for i := 0 to Count - 1 do
		if UpperCase(GetKey(i)) = UpperCase(Key) then Result := i;
end;

function TTupleList.GetKey(const Index: Integer): String;
begin
	if Index < Count then
		Result := Copy(Strings[Index],1,Pos('=',Strings[Index])-1)
	else
		Result := '';
end;

function TTupleList.GetInt(const Key: String): LongInt;
begin
	Result := StrToInt(Values[Key]);
end;

procedure TTupleList.ParseKeys(const Params: String; const Delimit: Char);
var
	i: Integer;
  Pair: String;
begin
	Pair := '';
	for i := 1 to Length(Params) do
  	if Params[i] = Delimit then begin
     	if Length(Pair) > 0 then begin
        	Add(Pair);
        	Pair := '';
       end;
   end else
     	if Params[i] = '_' then Pair := Pair + ' '
     	else Pair := Pair + Params[i];

	if Length(Pair) > 0 then Add(Pair);

end;

{ TTupleGrid implementations }

procedure TTupleGrid.AddPair(const Key: String; const Value: String);
begin
	if Cells[0,RowCount - 1] <> '' then RowCount := RowCount + 1;
	Cells[0,RowCount - 1] := Key;
	Cells[1,RowCount - 1] := Value;
end;

procedure TTupleGrid.ShowPairs(const Pairs: TTupleList);
var
	i: Integer;
begin
	with Pairs do begin
		RowCount := Count + 1;
		Cells[0,0] := 'Key';
		Cells[1,0] := 'Value';
		for i := 0 to Count - 1 do begin
			Cells[0,i+1] := Keys[i];
			Cells[1,i+1] := Values[Keys[i]];
		end;
	end;
end;

procedure Register;
begin
  RegisterComponents('Samples', [TTupleGrid]);
end;

end.
